<?php

defined( 'ABSPATH' ) or die( 'No script kiddies please!' );

class Dracula_Enqueue {

	private static $instance = null;

	public function __construct() {
		add_action( 'wp_enqueue_scripts', array( $this, 'frontend_scripts' ), 999 );
		add_action( 'login_enqueue_scripts', array( $this, 'frontend_scripts' ) );

		add_action( 'admin_enqueue_scripts', array( $this, 'admin_scripts' ) );
	}

	public function frontend_scripts() {

//		wp_enqueue_style('g-f', '');

		wp_register_style( 'dracula-frontend', DRACULA_ASSETS . '/css/frontend.css', array(), DRACULA_VERSION );

		wp_style_add_data( 'dracula-frontend', 'rtl', 'replace' );

		$custom_css = $this->get_custom_css();
		wp_add_inline_style( 'dracula-frontend', $custom_css );

		// JS Scripts
		$deps = [
			'wp-element',
			'wp-components',
			'wp-i18n',
			'wp-util',
		];

		// By default, style id startWith dracula- ignored by dark mode.
		// that why we need to add dracula_ prefix to the selector where we don't want to ignore dark mode
		wp_register_style( 'dracula_sweetalert2', DRACULA_ASSETS . '/vendor/sweetalert2/sweetalert2.min.css', [], DRACULA_VERSION );
		wp_register_script( 'dracula-sweetalert2', DRACULA_ASSETS . '/vendor/sweetalert2/sweetalert2.min.js', [], DRACULA_VERSION, true );


		// Page Transition Animation
		wp_register_script( 'dracula-gsap', DRACULA_ASSETS . '/vendor/gsap.js', [], '3.12.2', true );

		if ( ! empty( dracula_get_settings( 'pageTransition' ) ) && 'none' !== dracula_get_settings( 'pageTransition' ) ) {
			$deps[] = 'dracula-gsap';
		}

		wp_register_script( 'dracula-dark-mode', DRACULA_ASSETS . '/js/dark-mode.js', [], DRACULA_VERSION );
		$deps[] = 'dracula-dark-mode';

		// Enqueue sweetalert2 if analytics and feedback enabled
		if ( dracula_get_settings( 'enableAnalytics' ) && dracula_get_settings( 'enableFeedback' ) ) {
			wp_enqueue_style( 'dracula_sweetalert2' );
			wp_enqueue_script( 'dracula-sweetalert2' );
		}

		// Enqueue jquery-ui-draggable if draggable toggle enabled
		if ( dracula_get_settings( 'draggableToggle', false ) ) {
			$deps[] = 'jquery-ui-draggable';
		}

		wp_register_script( 'dracula-frontend', DRACULA_ASSETS . '/js/frontend.js', $deps, DRACULA_VERSION, true );
		wp_localize_script( 'dracula-frontend', 'dracula', $this->get_localize_data() );

		$is_active              = dracula_get_settings( 'frontendDarkMode', true ) && ! dracula_page_excluded();
		$is_active_reading_mode = dracula_get_settings( 'readingMode' ) && ! dracula_reading_mode_excluded();

		// Live Edit Scripts
		if ( dracula_is_elementor_editor_page() ) {
			$is_live_edit = current_user_can( 'manage_options' ) && ( $is_active || ! empty( $_GET['dracula-live-edit'] ) );

			if ( $is_live_edit || dracula_is_elementor_editor_page() ) {
				$this->enqueue_live_edit_scripts();
			}
		}

		// Frontend Scripts
		if ( $is_active || $is_active_reading_mode ) {
			wp_enqueue_style( 'dracula-frontend' );
			wp_enqueue_script( 'dracula-frontend' );
		}

	}

	public function admin_scripts( $hook ) {

		// Check if user can access dracula pages
		if ( ! dracula_is_user_dark_mode() && ! dracula_is_block_editor_page() ) {
			return;
		}

		if ( ! class_exists( 'Dracula_Admin' ) ) {
			require_once DRACULA_INCLUDES . '/class-admin.php';
		}

		$admin_pages = Dracula_Admin::instance()->get_admin_pages();

		// By default, style id startWith dracula- ignored by dark mode.
		// that why we need to add dracula_ prefix to the selector where we don't want to ignore dark mode
		wp_register_style( 'dracula_sweetalert2', DRACULA_ASSETS . '/vendor/sweetalert2/sweetalert2.min.css', [], DRACULA_VERSION );

		// Ignore toggle styles from dark mode
		wp_register_style( 'dracula-toggle', DRACULA_ASSETS . '/css/toggle.css', array(), DRACULA_VERSION );

		wp_enqueue_style( 'dracula_admin', DRACULA_ASSETS . '/css/admin.css', array(
			'wp-components',
			'dracula-toggle',
			'dracula_sweetalert2'
		), DRACULA_VERSION );

		wp_style_add_data( 'dracula_admin', 'rtl', 'replace' );

		// Javascript Dependencies
		$deps = [
			'wp-components',
			'wp-element',
			'wp-editor',
			'wp-util',
		];

		wp_register_script( 'dracula-dark-mode', DRACULA_ASSETS . '/js/dark-mode.js', [], DRACULA_VERSION );
		$deps[] = 'dracula-dark-mode';

		// If block editor page and !active return
		$block_editor_dark_mode = dracula_get_settings( 'blockEditorDarkMode', true );
		if ( ! $block_editor_dark_mode && dracula_is_block_editor_page() ) {
			$deps = array_diff( $deps, [ 'dracula-dark-mode' ] );
		}

		// Analytics page scripts
		if ( ! empty( $admin_pages['analytics'] ) && $admin_pages['analytics'] === $hook ) {
			wp_register_script( 'dracula-chart', DRACULA_ASSETS . '/vendor/chart.js', array( 'jquery-ui-datepicker' ), DRACULA_VERSION, true );
			$deps[] = 'dracula-chart';
		}

		wp_register_script( 'dracula-sweetalert2', DRACULA_ASSETS . '/vendor/sweetalert2/sweetalert2.min.js', [], DRACULA_VERSION, true );
		$deps[] = 'dracula-sweetalert2';

		// Settings Page
		if ( $admin_pages['dracula'] === $hook ) {
			wp_register_script( 'dracula-gsap', DRACULA_ASSETS . '/vendor/gsap.js', [], '3.12.2', true );
			$deps[] = 'dracula-gsap';
		}

		// Enqueue media scripts for settings and toggle builder page
		if ( in_array( $hook, [ $admin_pages['settings'], $admin_pages['toggle_builder'] ] ) ) {
			wp_enqueue_media();
		}

		// CSS Editor Scripts
		if ( $admin_pages['dracula'] === $hook || dracula_is_block_editor_page() || dracula_is_classic_editor_page() ) {
			wp_enqueue_script( 'wp-theme-plugin-editor' );
			wp_enqueue_style( 'wp-codemirror' );

			$cm_settings = [
				'codeEditor' => wp_enqueue_code_editor( array(
					'type'  => 'text/css',
					'theme' => 'dracula',
				) ),
			];

			wp_localize_script( 'dracula-admin', 'cm_settings', $cm_settings );
		}

		wp_enqueue_script( 'dracula-admin', DRACULA_ASSETS . '/js/admin.js', $deps, DRACULA_VERSION, true );
		wp_localize_script( 'dracula-admin', 'dracula', $this->get_localize_data( $hook ) );

	}

	public function enqueue_live_edit_scripts() {
		wp_enqueue_style( 'dracula-live-edit', DRACULA_ASSETS . '/css/live-edit.css', [
			'dashicons',
			'wp-components',
			'dracula-frontend',
			'dracula_sweetalert2',
			'wp-codemirror'
		], DRACULA_VERSION );

		wp_enqueue_media();
		wp_enqueue_script( 'dracula-sweetalert2' );
		wp_enqueue_script( 'dracula-gsap' );
		wp_enqueue_script( 'jquery-ui-draggable' );
		wp_enqueue_script( 'wp-theme-plugin-editor' );

		$cm_settings = [
			'codeEditor' => wp_enqueue_code_editor( array(
				'type'  => 'text/css',
				'theme' => 'dracula',
			) ),
		];

		wp_localize_script( 'dracula-frontend', 'cm_settings', $cm_settings );

		wp_enqueue_script( 'dracula-live-edit', DRACULA_ASSETS . '/js/live-edit.js', [
			'wp-editor',
			'wp-components',
			'dracula-frontend'
		], DRACULA_VERSION, true );
	}

	public function get_localize_data( $hook = false ) {

		$data = array(
			'homeUrl'    => home_url(),
			'ajaxUrl'    => admin_url( 'admin-ajax.php' ),
			'pluginUrl'  => DRACULA_URL,
			'settings'   => dracula_get_settings(),
			'isPro'      => true,
			'upgradeUrl' => false,
			'nonce'      => wp_create_nonce( 'dracula' ),
		);

		if ( is_admin() ) {
			$admin_pages = Dracula_Admin::instance()->get_admin_pages();

			if ( $admin_pages['dracula'] === $hook ) {
				$data['menus']              = dracula_get_menus();
				$data['userRoles']          = dracula_get_user_roles();
				$data['excludeList']        = dracula_get_exclude_list();
				$data['excludeReadingList'] = dracula_get_exclude_reading_list();
				$data['excludeTaxList']     = dracula_get_exclude_taxonomy_list();
				$data['showReviewPopup']    = current_user_can( 'manage_options' ) && 'off' != get_option( 'dracula_rating_notice' ) && 'off' != get_transient( 'dracula_rating_notice_interval' );
			}

			// check current user role
			global $current_user;
			$data['currentUserRole'] = ( ! empty( $current_user ) && ! empty( $current_user->roles ) ) ? $current_user->roles[0] : '';
		}

		$is_active    = dracula_get_settings( 'frontendDarkMode', true ) && ! dracula_page_excluded();
		$is_live_edit = current_user_can( 'manage_options' ) && ( $is_active || ! empty( $_GET['dracula-live-edit'] ) );

		$is_editor = dracula_is_block_editor_page() || dracula_is_classic_editor_page() || dracula_is_elementor_editor_page();

		if ( $is_live_edit || $is_editor ) {
			$data['menus'] = dracula_get_menus();

			$data['editDetails'] = dracula_get_edit_details__premium_only( $is_editor );
		}

		$page = ! empty( $_GET['page'] ) ? sanitize_key( $_GET['page'] ) : '';

		if ( $page == 'dracula' ) {
			$data['postTypes'] = dracula_get_post_type_list();
		}

		return $data;
	}

	/**
	 * Custom css
	 */
	public function get_custom_css() {
		$custom_css = '';

		// General Button
		$buttonAlignment = dracula_get_settings( 'buttonAlignment', 'start' );
		$button_variable = sprintf( '--reading-mode-button-alignment: %s !important;', $buttonAlignment );
		$custom_css      .= sprintf( '.reading-mode-buttons { %s }', $button_variable );

		// Reading Mode CSS Variable
		$readingModeBGColor   = dracula_get_settings( 'readingModeBGColor', '#E3F5FF' );
		$readingModeBGDarker  = dracula_color_brightness( $readingModeBGColor, - 30 );
		$readingModeTextColor = dracula_get_settings( 'readingModeTextColor', '#2F80ED' );

		$dracula_variable = '';
		$dracula_variable .= ! empty( $readingModeBGColor ) ? sprintf( '--reading-mode-bg-color: %s;', $readingModeBGColor ) : '';
		$dracula_variable .= ! empty( $readingModeBGColor ) ? sprintf( '--reading-mode-bg-darker: %s;', $readingModeBGDarker ) : '';
		$dracula_variable .= ! empty( $readingModeTextColor ) ? sprintf( '--reading-mode-text-color: %s;', $readingModeTextColor ) : '';

		$custom_css .= sprintf( '.reading-mode-buttons .reading-mode-button { %s }', $dracula_variable );

		// Time CSS Variable
		$timeBGColor   = dracula_get_settings( 'timeBGColor' );
		$timeBGDarker  = dracula_color_brightness( $timeBGColor, - 30 );
		$timeTextColor = dracula_get_settings( 'timeTextColor' );

		$time_variable = '';
		$time_variable .= ! empty( $timeBGColor ) ? sprintf( '--time-bg-color: %s;', $timeBGColor ) : '';
		$time_variable .= ! empty( $timeBGColor ) ? sprintf( '--time-bg-darker: %s;', $timeBGDarker ) : '';
		$time_variable .= ! empty( $timeTextColor ) ? sprintf( '--time-text-color: %s;', $timeTextColor ) : '';

		$custom_css .= sprintf( '.reading-mode-buttons .reading-mode-time { %s }', $time_variable );

		// Progressbar CSS Variable
		$progressbar_height = dracula_get_settings( 'progressbarHeight', '7' );
		$progressbar_color  = dracula_get_settings( 'progressbarColor', '#7C7EE5' );

		$progressbar_variable = '';
		$progressbar_variable .= sprintf( '--reading-mode-progress-height: %spx;', $progressbar_height );
		$progressbar_variable .= sprintf( '--reading-mode-progress-color: %s;', $progressbar_color );

		$custom_css .= sprintf( '.reading-mode-progress { %s }', $progressbar_variable );

		// Image Settings
		$invert_images  = dracula_get_settings( 'invertImages', false );
		$low_brightness = dracula_get_settings( 'lowBrightnessImages', false );
		$gray_scale     = dracula_get_settings( 'grayscaleImages', false );

		if ( $invert_images || $low_brightness || $gray_scale ) {
			$custom_css .= 'html[data-dracula-scheme="dark"] img:not(.dracula-toggle *, .dracula-ignore, .dracula-ignore * , .elementor-background-overlay, .elementor-element-overlay, .elementor-button-link, .elementor-button-link *, .elementor-widget-spacer, .elementor-widget-spacer *, .wp-block-button__link, .wp-block-button__link *){';

			$filter_css = '';

			if ( $invert_images ) {
				$invert_images_level = dracula_get_settings( 'invertImagesLevel', 80 ) / 100;
				$filter_css          .= sprintf( 'invert(%s) ', $invert_images_level );
			}
			if ( $low_brightness ) {
				$low_brightness_level = dracula_get_settings( 'lowBrightnessLevel', 80 ) / 100;
				$filter_css           .= sprintf( 'brightness(%s) ', $low_brightness_level );
			}
			if ( $gray_scale ) {
				$gray_scale_level = dracula_get_settings( 'grayscaleImagesLevel', 80 ) / 100;
				$filter_css       .= sprintf( 'grayscale(%s) ', $gray_scale_level );
			}

			$custom_css .= sprintf( 'filter: %s; }', $filter_css );

			$custom_css .= '}';
		}

		// Video Settings
		$video_low_brightness = dracula_get_settings( 'lowBrightnessVideos', false );
		$video_gray_scale     = dracula_get_settings( 'grayscaleVideos', false );

		if ( $video_low_brightness || $video_gray_scale ) {
			$custom_css .= 'html[data-dracula-scheme="dark"] video:not(.dracula-toggle *, .dracula-ignore, .dracula-ignore * ),';
			$custom_css .= 'html[data-dracula-scheme="dark"] iframe[src*="youtube.com"],';
			$custom_css .= 'html[data-dracula-scheme="dark"] iframe[src*="vimeo.com"],';
			$custom_css .= 'html[data-dracula-scheme="dark"] iframe[src*="dailymotion.com"]{';

			$filter_css = '';
			if ( $video_low_brightness ) {
				$video_low_brightness_level = dracula_get_settings( 'videoBrightnessLevel', 80 ) / 100;
				$filter_css                 .= sprintf( 'brightness(%s) ', $video_low_brightness_level );
			}
			if ( $video_gray_scale ) {
				$video_gray_scale_level = dracula_get_settings( 'grayscaleVideosLevel', 80 ) / 100;
				$filter_css             .= sprintf( 'grayscale(%s) ', $video_gray_scale_level );
			}

			$custom_css .= sprintf( 'filter: %s; }', $filter_css );

			$custom_css .= '}';
		}

		//Menu toggle size css
		$menu_toggle_size     = dracula_get_settings( 'menuToggleSize', 'normal' );
		$menu_toggle_selector = '.dracula-toggle-wrap.menu-item .dracula-toggle';
		if ( in_array( $menu_toggle_size, [ 'small', 'large' ] ) ) {
			$custom_css .= sprintf( '%s{ --toggle-scale: %s; }', $menu_toggle_selector, 'small' == $menu_toggle_size ? '.8' : '1.5' );
		}

		// Toggle size css
		$toggle_size     = dracula_get_settings( 'toggleSize', 'normal' );
		$toggle_selector = '.dracula-toggle-wrap .dracula-toggle';
		if ( in_array( $toggle_size, [ 'small', 'large' ] ) ) {
			$custom_css .= sprintf( '%s{ --toggle-scale: %s; }', $toggle_selector, 'small' == $toggle_size ? '.8' : '1.5' );
		}

			// Toggle size css
			$toggle_size              = dracula_get_settings( 'toggleSize', 'normal' );
			$floating_toggle_selector = '.dracula-toggle-wrap.floating .dracula-toggle';
			if ( 'custom' == $toggle_size ) {
				$custom_toggle_size = dracula_get_settings( 'customToggleSize', 100 ) / 100;
				$custom_css         .= sprintf( '%s{ --toggle-scale: %s; }', $floating_toggle_selector, $custom_toggle_size );
			}

			//Menu toggle size css
			$menu_toggle_size     = dracula_get_settings( 'menuToggleSize', 'normal' );
			$menu_toggle_selector = '.dracula-toggle-wrap.menu-item .dracula-toggle';
			if ( in_array( $menu_toggle_size, [ 'small', 'large' ] ) ) {
				$custom_css .= sprintf( '%s{ --toggle-scale: %s; }', $menu_toggle_selector, 'small' == $menu_toggle_size ? '.8' : '1.5' );
			} else if ( 'custom' == $menu_toggle_size ) {
				$custom_menu_toggle_size = dracula_get_settings( 'customMenuToggleSize', 100 ) / 100;
				$custom_css              .= sprintf( '%s{ --toggle-scale: %s; }', $menu_toggle_selector, $custom_menu_toggle_size );
			}

			// Toggle position css
			$position = dracula_get_settings( 'togglePosition', 'right' );

			if ( 'custom' == $position ) {
				$side   = dracula_get_settings( 'sideOffset', '30' );
				$bottom = dracula_get_settings( 'bottomOffset', '30' );

				$custom_css .= sprintf( '.dracula-toggle-wrap.position-custom{ --side-offset: %spx; --bottom-offset: %spx; }', $side, $bottom );
			}

			// Font size css
			if ( dracula_get_settings( 'changeFont', false ) ) {
				$font_size = dracula_get_settings( 'fontSize' );
				if ( 'custom' == $font_size ) {
					$font_size = dracula_get_settings( 'customFontSize', 100 ) / 100;
				}

				$custom_css .= sprintf( 'html[data-dracula-scheme="dark"] body>*:not(#dracula-live-edit){zoom: %s;}', $font_size );
			}

			$activeCustomPreset = dracula_get_settings( 'activeCustomPreset' );
			$customPresets      = dracula_get_settings( 'customPresets', [] );

			// check not empty customPresets
			if ( ! empty( $customPresets ) ) {
				$index                = array_search( $activeCustomPreset, array_column( $customPresets, 'id' ) );
				$currentCustomPresets = $customPresets[ $index ];

				// Link Color
				$link_color = $currentCustomPresets['colors']['link'];
				if ( ! empty( $link_color ) ) {
					$custom_css .= sprintf( 'html[data-dracula-scheme="dark"] a:not(.dracula-ignore *, [class*="btn"], [class*="button"]){color: %s!important;}', $link_color );

					$link_hover_color = dracula_color_brightness( $link_color, - 50 );
					$custom_css       .= sprintf( 'html[data-dracula-scheme="dark"] a:not(.dracula-ignore *, [class*="btn"], [class*="button"]):hover{color: %s!important;}', $link_hover_color );
				}

				// Button Color
				$button_bg_color   = $currentCustomPresets['colors']['button']['bg'];
				$button_text_color = $currentCustomPresets['colors']['button']['text'];

				$button_selector = 'button:not(.dracula-ignore *), input[type="button"]:not(.dracula-ignore *), input[type="submit"]:not(.dracula-ignore *), input[type="reset"]:not(.dracula-ignore *), a[class*="btn"]:not(.dracula-ignore *), a[class*="button"]:not(.dracula-ignore *)';
				if ( ! empty( $button_bg_color ) ) {
					$custom_css .= dracula_add_dark_mode_selector_prefix( sprintf( '%s{background-color: %s!important;}', $button_selector, $button_bg_color ) );

					$button_hover_bg_color = dracula_color_brightness( $button_bg_color, - 10 );
					$custom_css            .= dracula_add_dark_mode_selector_prefix( sprintf( 'button:not(.dracula-ignore *):hover, input[type="button"]:not(.dracula-ignore *):hover, input[type="submit"]:not(.dracula-ignore *):hover, input[type="reset"]:not(.dracula-ignore *):hover, a[class*="btn"]:not(.dracula-ignore *):hover, a[class*="button"]:not(.dracula-ignore *):hover{background-color: %s!important;}', $button_hover_bg_color ) );
				}

				if ( ! empty( $button_text_color ) ) {
					$custom_css .= dracula_add_dark_mode_selector_prefix( sprintf( '%s{color: %s!important;}', $button_selector, $button_text_color ) );
				}

				// Input Color
				$input_bg_color   = $currentCustomPresets['colors']['input']['bg'];
				$input_text_color = $currentCustomPresets['colors']['input']['text'];

				$input_selector = 'input:not(.dracula-ignore *, [type*="button"]), textarea:not(.dracula-ignore *), select:not(.dracula-ignore *)';

				if ( ! empty( $input_bg_color ) ) {
					$custom_css .= dracula_add_dark_mode_selector_prefix( sprintf( '%s{background-color: %s!important;}', $input_selector, $input_bg_color ) );
				}

				if ( ! empty( $input_text_color ) ) {
					$custom_css .= dracula_add_dark_mode_selector_prefix( sprintf( '%s{color: %s!important;}', $input_selector, $input_text_color ) );
				}
			}

			// Dark Mode Custom css
		$custom_css .= dracula_get_settings( 'compiledCss' );


		return $custom_css;
	}

	public static function instance() {
		if ( null === self::$instance ) {
			self::$instance = new self;
		}

		return self::$instance;
	}

}


Dracula_Enqueue::instance();
